use Test::More tests => 2;
use strict;
use warnings;

BEGIN {
use_ok( 'Object::Generic::False' );
use_ok( 'Object::Generic' );
}

# diag( 
# "Testing Object::Generic $Object::Generic::VERSION, Perl 5.008003, /usr/bin/perl" 
# );
